# Licensed Materials Property of IBM, (C) Copyright IBM Corp 1998, 2003
# Metronome.pl  Version: 1.00 for Perl 5

# MODULES----------------
use Socket;
use File::Path;
use File::Copy;
use File::Basename;

# CONSTANTS--------------
$MAESTRODIR='$(Install_dir)';

$STR_VAR_OS='OS';
$STR_VAR_HOSTNAME='HOSTNAME';
$STR_VAR_LOGIN='LOGIN';
$STR_VAR_MAESTROHOME='MAESTROHOME';
$STR_VAR_MAESTROUSERS='MAESTROUSERS';
$STR_VAR_MAESTROUSER='MAESTROUSER';
$STR_VAR_MAESTROUSER_UID='MAESTROUSER_UID';
$STR_VAR_MAESTROUSER_GID='MAESTROUSER_GID';
$STR_VAR_MAESTROUSER_PATH='MAESTROUSER_PATH';
$STR_VAR_MAESTROVERSION='MAESTROVERSION';
$STR_VAR_MAESTROTHISCPU='thiscpu';
$STR_VAR_MAESTROMASTERCPU='master';
$STR_VAR_CONMAN_SC='CONMAN_SC';
$STR_VAR_SYMPHONY='SYMPHONY';
$STR_VAR_DATECALC_SYMPHONYSCHEDDATE='DATECALC_SYMPHONYSCHEDDATE';
$STR_VAR_LOCALOPTS='LOCALOPTS';
$STR_VAR_LOCALOPTSNMPORT='nm port';
$STR_VAR_GLOBALOPTS='GLOBALOPTS';
$STR_VAR_UNISONHOME='UNISONHOME';
$STR_VAR_COMPONENTSFILE='COMPONENTSFILE';
$STR_VAR_PROCESSLIST='PROCESSLIST';
$STR_VAR_PROCESSBATCHMAN='PROCESSBATCHMAN';
$STR_VAR_PROCESSJOBMAN='PROCESSJOBMAN';
$STR_VAR_PROCESSMAILMAN='PROCESSMAILMAN';
$STR_VAR_PROCESSNETMAN='PROCESSNETMAN';
# $STR_VAR_PROCESSJOBMON='PROCESSJOBMON';
# $STR_VAR_PROCESSSTAGEMAN='PROCESSSTAGEMAN';
$STR_VAR_DATECALC_TODAY='DATECALC_TODAY';
$STR_VAR_MAILBOXFILE='MAILBOXMSGFILE';
$STR_VAR_INTERCOMFILE='INTERCOMMSGFILE';
$STR_VAR_COURIERFILE='COURIERMSGFILE';
$STR_VAR_NETREQFILE='NETREQMSGFILE';
$STR_VAR_ENVIRONMENT='ENVIRONMENT';

$STR_VAR_OS_WIN='MSWin32';
$STR_VAR_OS_LINUX='linux';
$ACTUALSNAPSHOTDIR='.';
@FILESTOPACKAGE=();
@VALID_OS=($STR_VAR_OS_WIN,$STR_VAR_OS_LINUX,$STR_VAR_OS_UNIX);
@MOSTIMPORTANT_VAR=(
	$STR_VAR_OS,
	$STR_VAR_HOSTNAME,
	$STR_VAR_LOGIN,
	$STR_VAR_MAESTROHOME,
	$STR_VAR_MAESTROUSER,
	$STR_VAR_MAESTROUSER_UID,
	$STR_VAR_MAESTROUSER_GID,
	$STR_VAR_MAESTROUSER_PATH,
	$STR_VAR_MAESTROVERSION,
	$STR_VAR_MAESTROTHISCPU,
	$STR_VAR_MAESTROMASTERCPU,
	$STR_VAR_DATECALC_TODAY,
	$STR_VAR_SYMPHONY,
	$STR_VAR_DATECALC_SYMPHONYSCHEDDATE,
	$STR_VAR_LOCALOPTS,
	$STR_VAR_LOCALOPTSNMPORT,
	$STR_VAR_GLOBALOPTS,
	$STR_VAR_UNISONHOME,
	$STR_VAR_MAESTROUSERS,
	$STR_VAR_COMPONENTSFILE,
	$STR_VAR_PROCESSBATCHMAN,
	$STR_VAR_PROCESSJOBMAN,
	$STR_VAR_PROCESSMAILMAN,
	$STR_VAR_PROCESSNETMAN,
	$STR_VAR_MAILBOXFILE,
	$STR_VAR_INTERCOMFILE,
	$STR_VAR_COURIERFILE,
	$STR_VAR_NETREQFILE
	);

# GLOBAL VARIABLES-------
%envHashTable=();

# BEGIN------------------
print("Metronome COPYRIGHT  2002 by IBM Corporation.
-*Args*-
	-pack	 create package for support
	-html    generate html report
	-prall	 print all variables
");

if (!validateArguments())
{exit 0;}

fillEnvHashTableFromArguments();
fillEnvHashTableFromSystem();
fillEnvHashTableFromConman();
fillEnvHashTableFromOptsFiles();
fillEnvHashTableFromSymphony();
fillEnvHashTableFromComponentsFile();
fillEnvHashTableFromMsgFiles();

$MaestroHomeDir=$envHashTable{$STR_VAR_MAESTROHOME};
@FILESTOPACKAGE=(
		"$envHashTable{$STR_VAR_MAESTROHOME}/Symphony",
		"$envHashTable{$STR_VAR_MAESTROHOME}/localopts",
		"$envHashTable{$STR_VAR_MAESTROHOME}/mozart/globalopts",
		"$envHashTable{$STR_VAR_MAESTROHOME}/version/maestro.info",
		"$envHashTable{$STR_VAR_MAESTROHOME}/version/patch.info",
		"$envHashTable{$STR_VAR_MAESTROHOME}/Mailbox.msg",
		"$envHashTable{$STR_VAR_MAESTROHOME}/Intercom.msg",
		"$envHashTable{$STR_VAR_MAESTROHOME}/Courier.msg",
		"$envHashTable{$STR_VAR_MAESTROHOME}/NetReq.msg",
		"$envHashTable{$STR_VAR_MAESTROHOME}/schedlog"
		);
#	Is packed also the most recent dir in "$envHashTable{$STR_VAR_MAESTROHOME}/stdlist"

if (isArgument('-pack'))
{preparePackage();}

if (isArgument('-html'))
{printHtmlReport();}

if (isArgument('-prall'))
{printHashTable(@envHashTable);}

print "\n\n";
printFilteredHashTable();

exit 0;


# --------------------------------------------------
# input parameter void
# output parameter 0,1
# --------------------------------------------------
sub validateArguments()
{
	local(@envTableKeys);
	local(@tokens);
	local($varName);
	local($varValue);

	# Printing parameters
	if ($#ARGV>-1)
	{
		print("Running with args:\n");
		for ($i=0; $i<=$#ARGV; $i++)
		{
			print "$i] $ARGV[$i]\n";
		}
		print "\n";
	}

	return 1;
}

# --------------------------------------------------
# input parameter String
# output parameter 0,1
# --------------------------------------------------
sub isArgument()
{
	local($arg);
	$arg=$_[0];

	if ($#ARGV>-1)
	{
		for ($i=0; $i<=$#ARGV; $i++)
		{
		  if ($ARGV[$i] eq $arg)
		  {return 1;}
		}
	}
	return 0;
}

# --------------------------------------------------
# input parameter void
# output parameter void
# --------------------------------------------------
sub fillEnvHashTableFromArguments()
{
	local(@envTableKeys);
	local(@tokens);
	local($varName);
	local($varValue);

	if ($#ARGV>-1)
	{
		for ($i=0; $i<=$#ARGV; $i++)
		{
			@tokens = split(/=/,$ARGV[$i]);
			$varName=&trim($tokens[0]);
			if ($#tokens==1)
			{
				$varValue=&trim($tokens[1]);
				$envHashTable{$varName}=$varValue;
			}
		}
	}
}

# --------------------------------------------------
# input parameter String
# output parameter void
# --------------------------------------------------
sub fillEnvHashTableFromString()
{
	local(@tokens);

	# Tokenizing string
	@tokens = split(/=/,$_[0]);

	if ($#tokens==1)
	{
		$tokens[0]=&trim($tokens[0]);
		$tokens[1]=&trim($tokens[1]);
		$envHashTable{$tokens[0]}=$envHashTable{$tokens[0]} || $tokens[1];
	}
}

# --------------------------------------------------
# input parameter void
# output parameter void
# --------------------------------------------------
sub fillEnvHashTableFromSystem()
{
	local($temp);
	local($com);
    local(@tokens);

	print "Accessing system info ...";

    $envHashTable{$STR_VAR_HOSTNAME}=$envHashTable{$STR_VAR_HOSTNAME} || getHostName();
    $envHashTable{$STR_VAR_LOGIN}=getlogin || getpwuid($<) || 'Unknown';
    $envHashTable{$STR_VAR_OS}=$envHashTable{$STR_VAR_OS} || $^O;

    $envHashTable{$STR_VAR_MAESTROHOME}=$envHashTable{$STR_VAR_MAESTROHOME} || getMaestroDir();

    # All maestro users
    $envHashTable{$STR_VAR_MAESTROUSERS}=getAllMaestroUsers();

	$envHashTable{$STR_VAR_MAESTROUSER}=$envHashTable{$STR_VAR_MAESTROUSER} || getMaestroUser();

    $envHashTable{$STR_VAR_UNISONHOME}=$envHashTable{$STR_VAR_UNISONHOME} || "$envHashTable{$STR_VAR_MAESTROHOME}/../unison";

    # maestro user details
	fillEnvHashTableWithMaestroUserDetails();

	# process list
	$envHashTable{$STR_VAR_PROCESSLIST}=getProcessList();

    # maestro processes details
	fillEnvHashTableWithMaestroProcessDetail();

	$temp=`"$envHashTable{$STR_VAR_MAESTROHOME}/bin/datecalc" today pic mm/dd/yy`;
	chop($temp);
    $envHashTable{$STR_VAR_DATECALC_TODAY}=$temp;

    fillEnvHashTableFromEnvironment();

    $INC[$#INC+1]="$envHashTable{$STR_VAR_MAESTROHOME}/bin";

    print "[OK]\n";
}

# --------------------------------------------------
# input parameter void
# output parameter void
# --------------------------------------------------
sub fillEnvHashTableFromConman()
{
	local($conman);
	local($maestrouser_path);
    local(@tokens);
    local($tokenIndex);

    print "Accessing conman info ...\n";

	$maestrouser_path=$envHashTable{$STR_VAR_MAESTROHOME};
	# $conman="\"$maestrouser_path/bin/conman\" -v";
	$conman="$maestrouser_path/bin/conman -v";
	$conman=`$conman`;

    @tokens=split(/\s+/,$conman);

	for ($i=0; $i<=$#tokens; $i++)
	{
		if ($tokens[$i]=~ /.*CONMAN.*/)
		{$tokenIndex=$i+1;$i=$#tokens;}
	}

	# maestro version
	$envHashTable{$STR_VAR_MAESTROVERSION}=@tokens[$tokenIndex] || 'Unknown';

	#Collecting cpus
	local($write);
	local($info);
	$conman="$maestrouser_path/bin/conman sc";
	$conman=`$conman`;
    $write=0;
	@tokens=split(/\n/,$conman);

	for ($i=0; $i<=$#tokens; $i++)
	{
            if ($tokens[$i]=~ /^sc/)
	    {
	    	$write=1;
	    }
	    else
	   {
	     	if ($write)
	     	{
	     		$info=$info.$tokens[$i]."\n";
			}
		}
 	}
 	$envHashTable{$STR_VAR_CONMAN_SC}=$info;

	print "[OK]\n";
}

# --------------------------------------------------
# input parameter void
# output parameter void
# --------------------------------------------------
sub fillEnvHashTableFromEnvironment()
{
    local($temp);
	local(@keys);
	local(@values);
	@keys = keys %ENV;
    @values = values %ENV;
    while (@keys)
	{
        $temp=$temp.pop(@keys).'='.pop(@values)."\n";
    }
    $envHashTable{$STR_VAR_ENVIRONMENT}=$temp;
}

# --------------------------------------------------
# input parameter void
# output parameter void
# --------------------------------------------------
sub fillEnvHashTableWithMaestroProcessDetail()
{
    local($temp);
    local(@riga);
    local(@tokens);
	local($BATCHMANFOUND);
	local($JOBMANFOUND);
	local($MAILMANFOUND);
	local($NETMANFOUND);
	# local($JOBMONFOUND);
    # local($STAGEMANFOUND);
    local($PIDBATCHMAN);
    local($PIDJOBMAN);
    local($PIDMAILMAN);
    local($PIDNETMAN);

    $PIDBATCHMAN='Not found';
    $PIDJOBMAN='Not found';
    $PIDMAILMAN='Not found';
    $PIDNETMAN='Not found';
	$BATCHMANFOUND=FALSE;
	$JOBMANFOUND=FALSE;
	$MAILMANFOUND=FALSE;
	$NETMANFOUND=FALSE;
	# $JOBMONFOUND=FALSE;
    # $STAGEMANFOUND=FALSE;

        $temp="$envHashTable{$STR_VAR_MAESTROHOME}/pids/BATCHMAN.pid";
	if (-e $temp)
	{
		open (FILEIN, "< $temp") || print "\n**** Can't open file $temp\n\n";
		@riga = <FILEIN>;
        	$PIDBATCHMAN=trim($riga[0]);
		close(FILEIN1);
	}

        $temp="$envHashTable{$STR_VAR_MAESTROHOME}/pids/JOBMAN.pid";
	if (-e $temp)
	{
		open (FILEIN, "< $temp") || print "\n**** Can't open file $temp\n\n";
		@riga = <FILEIN>;
        	$PIDJOBMAN=trim($riga[0]);
		close(FILEIN1);
	}

	$temp="$envHashTable{$STR_VAR_MAESTROHOME}/pids/MAILMAN.pid";
	if (-e $temp)
	{
		open (FILEIN, "< $temp") || print "\n**** Can't open file $temp\n\n";
		@riga = <FILEIN>;
        	$PIDMAILMAN=trim($riga[0]);
		close(FILEIN1);
	}

        $temp="$envHashTable{$STR_VAR_MAESTROHOME}/pids/NETMAN.pid";

	if (-e $temp)
	{
		open (FILEIN, "< $temp") || print "\n**** Can't open file $temp\n\n";
		@riga = <FILEIN>;
                $PIDNETMAN=trim($riga[0]);
		close(FILEIN1);
	}
	else
	{
            $temp="$envHashTable{$STR_VAR_UNISONHOME}/netman/pids/NETMAN.pid";

            if (-e $temp)
            {
            	open (FILEIN, "< $temp") || print "\n**** Can't open file $temp\n\n";
        	@riga = <FILEIN>;
                $PIDNETMAN=trim($riga[0]);
        	close(FILEIN1);
            }
        }

    # maestro processes details
	@tokens = split(/\n/,$envHashTable{$STR_VAR_PROCESSLIST});

	for ($i=0; $i<=$#tokens; $i++)
	{
                if (uc($tokens[$i])=~ /^[.\s]*$PIDBATCHMAN.*BATCHMAN/)
	 	{$BATCHMANFOUND='TRUE';}
	 	if (uc($tokens[$i])=~ /^[.\s]*$PIDJOBMAN.*JOBMAN/)
	 	{$JOBMANFOUND='TRUE';}
	 	if (uc($tokens[$i])=~ /^[.\s]*$PIDMAILMAN.*MAILMAN/)
	 	{$MAILMANFOUND='TRUE';}
	 	if (uc($tokens[$i])=~ /^[.\s]*$PIDNETMAN.*NETMAN/)
	 	{$NETMANFOUND='TRUE';}
	 	# if (uc($tokens[$i])=~ /^[.\s]*$PIDJOBMON.*JOBMON/)
	 	# {$JOBMONFOUND='TRUE';}
	 	# if (uc($tokens[$i])=~ /^[.\s]*$PIDSTAGEMAN.*STAGEMAN/)
	 	# {$STAGEMANFOUND='TRUE';}
	}

    if ($BATCHMANFOUND eq 'TRUE')
	{$envHashTable{$STR_VAR_PROCESSBATCHMAN}="LIVE $PIDBATCHMAN";}
	else
        {$envHashTable{$STR_VAR_PROCESSBATCHMAN}='DOWN';}
    if ($JOBMANFOUND eq 'TRUE')
	{$envHashTable{$STR_VAR_PROCESSJOBMAN}="LIVE $PIDJOBMAN";}
	else
        {$envHashTable{$STR_VAR_PROCESSJOBMAN}='DOWN';}
    if ($MAILMANFOUND eq 'TRUE')
	{$envHashTable{$STR_VAR_PROCESSMAILMAN}="LIVE $PIDMAILMAN";}
	else
        {$envHashTable{$STR_VAR_PROCESSMAILMAN}='DOWN';}
    if ($NETMANFOUND eq 'TRUE')
	{$envHashTable{$STR_VAR_PROCESSNETMAN}="LIVE $PIDNETMAN";}
	else
    	{$envHashTable{$STR_VAR_PROCESSNETMAN}='DOWN';}
    #if ($STAGEMANFOUND eq 'TRUE')
	#   {$envHashTable{$STR_VAR_PROCESSSTAGEMAN}="LIVE $PIDSTAGEMAN";}
	#else
    	#   {$envHashTable{$STR_VAR_PROCESSSTAGEMAN}='DOWN';}
    #if ($JOBMONFOUND eq 'TRUE')
	 	#   {$envHashTable{$STR_VAR_PROCESSJOBMON}="LIVE $PIDJOBMON";}
	#else
         #   {$envHashTable{$STR_VAR_PROCESSJOBMON}='DOWN';}

}

# --------------------------------------------------
# input parameter void
# output parameter void
# --------------------------------------------------
sub fillEnvHashTableFromSymphony()
{
	local($SYMPHONYPATH);
	local($temp);

	print "Accessing symphony ...";

	$SYMPHONYPATH="$envHashTable{$STR_VAR_MAESTROHOME}/Symphony";

	if (-e $SYMPHONYPATH)
	{
		$envHashTable{$STR_VAR_SYMPHONY}="Exist ".getDetailsOnFile($SYMPHONYPATH);
		$temp=`"$envHashTable{$STR_VAR_MAESTROHOME}/bin/datecalc" scheddate pic mm/dd/yy`;
		chop($temp);
        	$envHashTable{$STR_VAR_DATECALC_SYMPHONYSCHEDDATE}=$temp;
	}
	else
	{
		$envHashTable{$STR_VAR_SYMPHONY}="Not exist";
	}

	print "[OK]\n";
}

# --------------------------------------------------
# input parameter void
# output parameter void
# --------------------------------------------------
sub fillEnvHashTableFromOptsFiles()
{
	if (-e "$envHashTable{$STR_VAR_MAESTROHOME}/localopts")
	{
		$envHashTable{$STR_VAR_LOCALOPTS}="Exist ".getDetailsOnFile("$envHashTable{$STR_VAR_MAESTROHOME}/localopts");
		if (-r "$envHashTable{$STR_VAR_MAESTROHOME}/localopts")
		{fillEnvHashTableFromFile("$envHashTable{$STR_VAR_MAESTROHOME}/localopts");}
	}
	else {	$envHashTable{$STR_VAR_LOCALOPTS}="Not exist";   }

	if (-e "$envHashTable{$STR_VAR_MAESTROHOME}/mozart/globalopts")
	{
		$envHashTable{$STR_VAR_GLOBALOPTS}="Exist ".getDetailsOnFile("$envHashTable{$STR_VAR_MAESTROHOME}/mozart/globalopts");
		if (-r "$envHashTable{$STR_VAR_MAESTROHOME}/mozart/globalopts")
		{fillEnvHashTableFromFile("$envHashTable{$STR_VAR_MAESTROHOME}/mozart/globalopts");}
	}
	else {	$envHashTable{$STR_VAR_GLOBALOPTS}="Not exist";   }
}

# --------------------------------------------------
# input parameter void
# output parameter void
# --------------------------------------------------
sub fillEnvHashTableFromComponentsFile()
{
    	local($components);
    	print "Accessing components file ...";

    	$components="$envHashTable{$STR_VAR_UNISONHOME}/components";
	if (-e "$components")
	{
           $envHashTable{$STR_VAR_COMPONENTSFILE}="Exist ".getDetailsOnFile("$components");
           print "[OK]\n";
	}
	else
	{$envHashTable{$STR_VAR_COMPONENTSFILE}="Not exist";print "[SKIP]\n";}
}

# --------------------------------------------------
# input parameter void
# output parameter void
# --------------------------------------------------
sub fillEnvHashTableFromMsgFiles()
{
    	print "Accessing msg files ...";

	if (-e "$envHashTable{$STR_VAR_MAESTROHOME}/Mailbox.msg")
		{  $envHashTable{$STR_VAR_MAILBOXFILE}="Exist ".getDetailsOnFile("$envHashTable{$STR_VAR_MAESTROHOME}/Mailbox.msg");}
	else  {$envHashTable{$STR_VAR_MAILBOXFILE}="Not exist";}

    if (-e "$envHashTable{$STR_VAR_MAESTROHOME}/Intercom.msg")
		{  $envHashTable{$STR_VAR_INTERCOMFILE}="Exist ".getDetailsOnFile("$envHashTable{$STR_VAR_MAESTROHOME}/Intercom.msg");}
	else  {$envHashTable{$STR_VAR_INTERCOMFILE}="Not exist";}

    if (-e "$envHashTable{$STR_VAR_MAESTROHOME}/Courier.msg")
		{  $envHashTable{$STR_VAR_COURIERFILE}="Exist ".getDetailsOnFile("$envHashTable{$STR_VAR_MAESTROHOME}/Courier.msg");}
	else  {$envHashTable{$STR_VAR_COURIERFILE}="Not exist";}

    if (-e "$envHashTable{$STR_VAR_MAESTROHOME}/NetReq.msg")
		{  $envHashTable{$STR_VAR_NETREQFILE}="Exist ".getDetailsOnFile("$envHashTable{$STR_VAR_MAESTROHOME}/NetReq.msg");}
	else  {$envHashTable{$STR_VAR_NETREQFILE}="Not exist";}

	print "[OK]\n";
}

# --------------------------------------------------
# input parameter void
# output parameter void
# --------------------------------------------------
sub preparePackage()
{
    local($temp);
    local($timeStamp);
    local($dest);
    local($USuserName);

    print "Preparing package ...";

    $timeStamp=`"$envHashTable{$STR_VAR_MAESTROHOME}/bin/datecalc" today pic yyyymmddhhmm`;
    chop($timeStamp);
    $ACTUALSNAPSHOTDIR="$envHashTable{$STR_VAR_MAESTROHOME}/snapshots/snap_$timeStamp";

    mkpath("$ACTUALSNAPSHOTDIR",true,0777);

    $timeStamp=`"$envHashTable{$STR_VAR_MAESTROHOME}/bin/datecalc" today pic yyyy.mm.dd`;
	chop($timeStamp);
    $USuserName=$envHashTable{$STR_VAR_MAESTROUSER};

    $USuserName =~ s/^.*\\//;	# Remove domain
    $USuserName=uc($USuserName);

    #------------Adding limited content of stdlist dir
    #print "--PRINTING $envHashTable{$STR_VAR_MAESTROHOME}/stdlist* CONTENT\n";
    local(@STDLISTFILES);
    local($startIndex);
    @STDLISTFILES=glob("$envHashTable{$STR_VAR_MAESTROHOME}/stdlist/*");

    $startIndex=$#STDLISTFILES-3;
    if ($startIndex<0)
    {$startIndex=0;}

    for ($i=$startIndex; $i<=$#STDLISTFILES; $i++)
    {
    	$FILESTOPACKAGE[$#FILESTOPACKAGE+1]=$STDLISTFILES[$i];
    	#print "File name is $STDLISTFILES[$i]\n";
    }
    #------------End Adding limited content of stdlist dir

    for ($i=0; $i<=$#FILESTOPACKAGE; $i++)
    {
    	#print "Ana $FILESTOPACKAGE[$i]\n";
    	$dest="$ACTUALSNAPSHOTDIR/".basename($FILESTOPACKAGE[$i]);
     	if (-d $FILESTOPACKAGE[$i])
     	{
     		local($destTarget);
		if ($envHashTable{$STR_VAR_OS} eq $STR_VAR_OS_WIN)
		{
			local($source);
        	        $source=$FILESTOPACKAGE[$i];
		   	$source =~ s/\//\\/g;
		   	$destTarget= "$ACTUALSNAPSHOTDIR/".basename($FILESTOPACKAGE[$i]).'/';
                        $destTarget =~ s/\//\\/g;
        	        $temp="xcopy $source $destTarget /E /R /Y";
        	        #print "CMD $temp\n";

        	        $temp=`$temp`;
        	        #print "OUTPUT $temp\n";
	 	}
	 	else
	 	{
        	        # $temp="tar -cvf $ACTUALSNAPSHOTDIR/archive"."$i".".tar $FILESTOPACKAGE[$i]";
        	        $destTarget= "$ACTUALSNAPSHOTDIR/".basename($FILESTOPACKAGE[$i]).'/';
        	        $temp="cp -R $FILESTOPACKAGE[$i] $destTarget";
        	        # print "CMD $temp\n";
        	        $temp=`$temp`;
        	        #print "OUTPUT $temp\n";
		}
	}
	else
	{
	    copy($FILESTOPACKAGE[$i],$dest);
	}
    }

    # Dumping database
    $temp="$envHashTable{$STR_VAR_MAESTROHOME}/bin/composer create $ACTUALSNAPSHOTDIR/calendars.def from calendars";
    $temp=`$temp`;
    $temp="$envHashTable{$STR_VAR_MAESTROHOME}/bin/composer create $ACTUALSNAPSHOTDIR/calendars.def from calendars";
    $temp=`$temp`;
    $temp="$envHashTable{$STR_VAR_MAESTROHOME}/bin/composer create $ACTUALSNAPSHOTDIR/job.def from job=@#@";
    $temp=`$temp`;
    $temp="$envHashTable{$STR_VAR_MAESTROHOME}/bin/composer create $ACTUALSNAPSHOTDIR/sched.def from sched=@#@";
    $temp=`$temp`;
    $temp="$envHashTable{$STR_VAR_MAESTROHOME}/bin/composer create $ACTUALSNAPSHOTDIR/prompts.def from prompt";
    $temp=`$temp`;
    $temp="$envHashTable{$STR_VAR_MAESTROHOME}/bin/composer create $ACTUALSNAPSHOTDIR/resource.def from resource";
    $temp=`$temp`;
    $temp="$envHashTable{$STR_VAR_MAESTROHOME}/bin/composer create $ACTUALSNAPSHOTDIR/parms.def from parms";
    $temp=`$temp`;
    $temp="$envHashTable{$STR_VAR_MAESTROHOME}/bin/composer create $ACTUALSNAPSHOTDIR/cpu.def from cpu=@";
    $temp=`$temp`;
    $temp="$envHashTable{$STR_VAR_MAESTROHOME}/bin/composer create $ACTUALSNAPSHOTDIR/users.def from user=@#@";
    $temp=`$temp`;

    if (!($envHashTable{$STR_VAR_OS} eq $STR_VAR_OS_WIN))
    {
	$temp="tar -cvf $envHashTable{$STR_VAR_MAESTROHOME}/snapshots/snapshot"."$timeStamp".".tar $ACTUALSNAPSHOTDIR";
	# print "CMD $temp\n";
	$temp=`$temp`;
	# print "OUTPUT $temp\n";
    }

    print "Created in $ACTUALSNAPSHOTDIR [OK]\n";
}

# --------------------------------------------------
# input parameter file name
# output parameter void
# --------------------------------------------------
sub fillEnvHashTableFromFile()
{
	# Local variables declaration
	local($FILENAME_INPUT);
	local(@riga);

	$FILENAME_INPUT=$_[0];

	print "Accessing file $FILENAME_INPUT...";
	open (FILEIN, "< $FILENAME_INPUT") || print "\n**** Can't open file $FILENAME_INPUT\n\n";
	@riga = <FILEIN>;
	print "[OK]\n";

	for ($i=0; $i<=$#riga; $i++)
	{
		chop($riga[$i]);
		&fillEnvHashTableFromString($riga[$i]);

	}
	close(FILEIN1);
}

# --------------------------------------------------
# input parameter void
# output parameter String
# --------------------------------------------------
sub getWorkingDir()
{
	local($resWorkingDir);
	use Cwd;
	$resWorkingDir= cwd;
	return $resWorkingDir;
}

# --------------------------------------------------
# input parameter void
# output parameter String
# --------------------------------------------------
sub getMaestroDir()
{
	if ($MAESTRODIR eq '$(Install_dir)')
 	{	return getWorkingDir();}
 	else
 	{	return $MAESTRODIR;}
}

# --------------------------------------------------
# input parameter void
# output parameter String
# --------------------------------------------------
sub getHostName()
{
    use Sys::Hostname;
	local($resHostName);
	local($a);
    local($b);
    local($c);
    local($d);
	$resHostName=hostname();
    $address=gethostbyname($resHostName);
    ($a,$b,$c,$d)=unpack('C4',$address);
    $resHostName=gethostbyaddr($address, AF_INET);

	# $resHostName='Unknown';

    #$iaddr = inet_aton("127.1"); # or whatever address
    #$resHostName= gethostbyaddr($iaddr, AF_INET);
    # or going the other way
    # $straddr = inet_ntoa($iaddr);

	# if ($envHashTable{$STR_VAR_OS} eq $STR_VAR_OS_WIN)
	# {
	# 	$resHostName='winMachine';
	# }
	return "$resHostName [$a.$b.$c.$d]";
}


# --------------------------------------------------
# input parameter void
# output parameter String
# --------------------------------------------------

# Remember: Use the file TWSRegistry.dat from version 8.2 of maestro
sub getAllMaestroUsers()
{
	local($resAllMaestroUser);

	if ($envHashTable{$STR_VAR_OS} eq $STR_VAR_OS_WIN)
	{
		require Win32::Registry;import Win32::Registry;

		local($p);
		local($reg);
		local(@subKeys);
		local($type);
		local($value);
                local($noKeyInReg);

                $resAllMaestroUser= '';

		$p="SOFTWARE\\Unison Software, Inc.\\Unison Maestro";
		$noKeyInReg="1";
                $noKeyInReg=$::HKEY_LOCAL_MACHINE->Open($p,$reg);

                if ($noKeyInReg=="1")
		{
                    $reg->GetKeys(\@subKeys);
                }

		for ($i=0; $i<=$#subKeys; $i++)
		{
		 	$resAllMaestroUser=$resAllMaestroUser."$subKeys[$i]";

                        $::HKEY_LOCAL_MACHINE->Open($p."\\".$subKeys[$i],$reg) || print "\n**** Not found in registry $!";
                        $reg->QueryValueEx('InstallDirectory', $type, $value);
                        if ($value)
                        {
            		   $value =~ s/\\/\//g;
            	           $resAllMaestroUser=$resAllMaestroUser.'@'."$value";
            	           $resAllMaestroUser=$resAllMaestroUser."\n";
                        }
	        }

		$p="SOFTWARE\\IBM\\Tivoli Workload Scheduler";
		$noKeyInReg="1";
                $noKeyInReg=$::HKEY_LOCAL_MACHINE->Open($p,$reg);
                @subKeys=();

                if ($noKeyInReg=="1")
		{
                    $reg->GetKeys(\@subKeys);
                }

		for ($i=0; $i<=$#subKeys; $i++)
		{
		 	$resAllMaestroUser=$resAllMaestroUser."$subKeys[$i]";

                        $::HKEY_LOCAL_MACHINE->Open($p."\\".$subKeys[$i],$reg) || print "\n**** Not found in registry $!";
                        $reg->QueryValueEx('InstallDirectory', $type, $value);
                        if ($value)
                        {
            		   $value =~ s/\\/\//g;
            	           $resAllMaestroUser=$resAllMaestroUser.'@'."$value";
            	           $resAllMaestroUser=$resAllMaestroUser."\n";
                        }
	        }

		chop($resAllMaestroUser);
	}
	else
	{
		$resAllMaestroUser=`grep maestro /etc/passwd` || 'Unknown';
	}
	return $resAllMaestroUser;
}

# --------------------------------------------------
# input parameter void
# output parameter String
# --------------------------------------------------
sub getMaestroUser()
{
	local($maestroUser);
	local($maestroHome);
	local($temp);
    local(@tokens);
    local($tokenIndex);

	#Windows: setown -d maestro.exe
	#Unix:
	$maestroHome=$envHashTable{$STR_VAR_MAESTROHOME};

    $maestroUser='Unknown';

    if ($envHashTable{$STR_VAR_OS} eq $STR_VAR_OS_WIN)
    {
       	$temp="$maestroHome"."/bin/setown.exe -d $maestroHome/bin/maestro.exe";
	    $temp=`$temp`;
		chop($temp);
		@tokens=split(/\s/,$temp);
		for ($i=0; $i<=$#tokens; $i++)
		{
		    if ($tokens[$i]=~ /.*is.*/)
		    {$tokenIndex=$i+1;$i=$#tokens;}
		}
	}
	else
    {
       	$temp="ls -l $maestroHome/bin/maestro";
	    $temp=`$temp`;
		chop($temp);
		@tokens=split(/\s+/,$temp);
		# print "DEBUG $tokens[0], $tokens[1], $tokens[2]\n";
		$tokenIndex=2;
	}

    $maestroUser="$tokens[$tokenIndex]";

	#$temp=getAllMaestroUsers();
    #@tokens=split(/\n/,$temp);

    # I have to choose the one with home dir  = $STR_VAR_MAESTROHOME
	# Scanning user list
	#for ($i=0; $i<=$#tokens; $i++)
	#{
	#	if ($envHashTable{$STR_VAR_OS} eq $STR_VAR_OS_WIN)
	#	{
        #            @tokens2=split(/@/,$tokens[$i]);
	#
        #            if (uc("$tokens2[1]") eq uc($maestroHome))
        #            {
        #               local(@token3);
        #               @token3=split(/_/,$tokens2[0]);
        #               $maestroUser=$token3[2];
        #               $i=$#tokens; # exit for
	#	    }
	#	}
	#	else
	#	{
        #           @tokens2=split(/:/,$tokens[$i]);
        #            if ($tokens2[5] eq $maestroHome)
        #            {
        #               $maestroUser=$tokens2[0];
        #               $i=$#tokens; # exit for
	#	    }
	#	}
	#
  	#}
	return $maestroUser;
}

# --------------------------------------------------
# input parameter void
# output parameter void
# --------------------------------------------------
sub fillEnvHashTableWithMaestroUserDetails()
{
	local($temp);
	local($com);
	local($maestroUser);
    local(@tokens);

	$maestroUser=$envHashTable{$STR_VAR_MAESTROUSER};

	if ($envHashTable{$STR_VAR_OS} eq $STR_VAR_OS_WIN)
	{
		# maestro uid
        $envHashTable{$STR_VAR_MAESTROUSER_UID}= 'Unknown';

	    # maestro gid
        $envHashTable{$STR_VAR_MAESTROUSER_GID}= 'Unknown';

	    # maestro path
        $envHashTable{$STR_VAR_MAESTROUSER_PATH}= 'Unknown';
	}
	else
	{
        $com="grep \"^$maestroUser:\" /etc/passwd";
        $temp=`$com`;
        @tokens=split(/:/,$temp);

		# maestro uid
        $envHashTable{$STR_VAR_MAESTROUSER_UID}=$tokens[2] || 'Unknown';

	    # maestro gid
        $envHashTable{$STR_VAR_MAESTROUSER_GID}=$tokens[3] || 'Unknown';

	    # maestro path
        $envHashTable{$STR_VAR_MAESTROUSER_PATH}=$tokens[5] || $ENV{'TWS_TISDIR'} || 'Unknown';
	}
}

# --------------------------------------------------
# input parameter String file path
# output parameter String
# --------------------------------------------------
sub getDetailsOnFile()
{
    	use File::stat;
	local($resFileDetails);
	local($fileSize);
	local($temp);
	local($stats);

	$resFileDetails='';

	if (-e $_[0])
	{
	if ($envHashTable{$STR_VAR_OS} eq $STR_VAR_OS_WIN)
	{
		$stats=stat($_[0]);

		$temp=$stats->size;
	    $resFileDetails=$resFileDetails." size=$temp";
		$temp=$stats->mode;
	    $resFileDetails=$resFileDetails." mode=$temp";
		$temp=$stats->uid;
	    $resFileDetails=$resFileDetails." userId=$temp";
		$temp=$stats->atime;
	    $resFileDetails=$resFileDetails." lastAccess=$temp";
		$temp=$stats->mtime;
	    $resFileDetails=$resFileDetails." lastModify=$temp";
		$temp=$stats->blocks;
	    $resFileDetails=$resFileDetails." blocks=$temp";
	    $resFileDetails=$resFileDetails." $_[0]";
 	}
 	else
 	{
        $temp="ls -l $_[0]";
        $temp=`$temp`;
        chop($temp);
        $resFileDetails=$resFileDetails.$temp;
	}
 	}
	return 	$resFileDetails;
}

# --------------------------------------------------
# input parameter
# output parameter String
# --------------------------------------------------
sub getProcessList()
{
	local($resProcessList);

	if ($envHashTable{$STR_VAR_OS} eq $STR_VAR_OS_WIN)
	{
	   $resProcessList="$envHashTable{$STR_VAR_MAESTROHOME}/unsupported/listproc";
	}
	else
	{
        $resProcessList="ps -e "; # | grep $envHashTable{$STR_VAR_MAESTROUSER}
	}
	return `$resProcessList`;
}


# --------------------------------------------------
# input array of String
# output parameter void
# --------------------------------------------------
sub printArray()
{
	print "printArray($#_)\n";
	for ($i=0; $i<=$#_; $i++)
	{
		print "$i $_[$i]\n";
	}
}
# --------------------------------------------------
# input void
# output parameter void
# --------------------------------------------------
sub printHashTable()
{
	print "******************\n";

	local(@keys);
	local(@values);
	@keys = keys %envHashTable;
    @values = values %envHashTable;
    while (@keys)
	{
        print pop(@keys), '=', pop(@values), "\n";
    }
	print "******************\n";

}

# --------------------------------------------------
# input void
# output parameter void
# --------------------------------------------------
sub printFilteredHashTable()
{
	print "******************\n";

	for ($i=0; $i<=$#MOSTIMPORTANT_VAR; $i++)
	{
        print $MOSTIMPORTANT_VAR[$i], '=', $envHashTable{$MOSTIMPORTANT_VAR[$i]} , "\n";
    }
	print "******************\n\n";

    if (&isArgument('-solve'))
    {
		local(@temp);
   		require MetronomeKB;import MetronomeKB;
		@temp=solveProblems();
		if ($#temp>=0)
		{
			print "Suggestions:\n";
			for ($i=0; $i<=$#temp; $i++)
			{
		    	print $temp[$i]."\n";
		 	}
	 	}
	 	else
	 	{print "No suggestions.\n";}
	}
}

# --------------------------------------------------
# input void
# output parameter void
# --------------------------------------------------
sub printHtmlReport()
{
    local($HTML_MOSTIMPORTANT_VAR);
    local($HTML_ALLCOLLECTED_VAR);
    local($HTML_ALLCOLLECTED_FILES);
    local($HTML_NETWORKTOPOLOGY_VAR);
    local($HTML_SUGGESTION_VAR);
	local($HTML_TIMESTAMP_VAR);

    local($r);
    local($fileTemplate);
    local($fileReport);
    local(@tokens);

    print "writing html report ...";

    $fileTemplate="$envHashTable{$STR_VAR_MAESTROHOME}/bin/reportTempl.htm";
    $fileReport=$ACTUALSNAPSHOTDIR.'/report.htm';

    $HTML_TIMESTAMP_VAR=(gmtime)[3]."/".((gmtime)[4]+1)."/".(1900+(gmtime)[5]);

    for ($i=0; $i<=$#MOSTIMPORTANT_VAR; $i++)
    {
    	$HTML_MOSTIMPORTANT_VAR=$HTML_MOSTIMPORTANT_VAR.'<TR BGCOLOR="white" CLASS="TableRowColor">';

		$HTML_MOSTIMPORTANT_VAR=$HTML_MOSTIMPORTANT_VAR.'<TD ALIGN="right" VALIGN="top" WIDTH="1%"><FONT SIZE="-1">';
		$HTML_MOSTIMPORTANT_VAR=$HTML_MOSTIMPORTANT_VAR."$MOSTIMPORTANT_VAR[$i]";
		$HTML_MOSTIMPORTANT_VAR =~ s/\n/'<BR>'/g;
	    $HTML_MOSTIMPORTANT_VAR=$HTML_MOSTIMPORTANT_VAR.'</FONT></TD>';

		$HTML_MOSTIMPORTANT_VAR=$HTML_MOSTIMPORTANT_VAR.'<TD>';
		$HTML_MOSTIMPORTANT_VAR=$HTML_MOSTIMPORTANT_VAR."$envHashTable{$MOSTIMPORTANT_VAR[$i]}";
		$HTML_MOSTIMPORTANT_VAR =~ s/\n/'<BR>'/g;
		$HTML_MOSTIMPORTANT_VAR=$HTML_MOSTIMPORTANT_VAR.'</TD>';

         $HTML_MOSTIMPORTANT_VAR=$HTML_MOSTIMPORTANT_VAR.'</TR>';
    }

    @tokens = split(/\n/,$envHashTable{$STR_VAR_CONMAN_SC});
    for ($i=0; $i<=$#tokens; $i++)
    {
    	chop($tokens[$i]);

    	local(@tokens2);
        local($j);
        local($limit);
        local($RestStr);
		@tokens2 = split(/\s+/,"$tokens[$i]");

    	if ($tokens2[3] eq 'MASTER')
    	{$HTML_NETWORKTOPOLOGY_VAR=$HTML_NETWORKTOPOLOGY_VAR.'<TR BGCOLOR="lightyellow" CLASS="TableRowColor">';}
    	else
        {$HTML_NETWORKTOPOLOGY_VAR=$HTML_NETWORKTOPOLOGY_VAR.'<TR BGCOLOR="white" CLASS="TableRowColor">';}

        $limit=6;
        $RestStr='';
        for ($j=0; ($j<=$#tokens2) && ($j<=$limit); $j++)
    	{
     		if (!($tokens2[$j] eq 'MASTER') && !($tokens2[$j] eq 'FTA'))
    		{
				$HTML_NETWORKTOPOLOGY_VAR=$HTML_NETWORKTOPOLOGY_VAR.'<TD>';
				$HTML_NETWORKTOPOLOGY_VAR=$HTML_NETWORKTOPOLOGY_VAR."$tokens2[$j]";
		        $HTML_NETWORKTOPOLOGY_VAR=$HTML_NETWORKTOPOLOGY_VAR.'</TD>';
	     	}
	     	else
	     	{$limit=$limit+1;}
    	}

        $limit=$limit+1;
        for ($j=$limit;$j<=$#tokens2; $j++)
    	{
    	    $RestStr=$RestStr.' '."$tokens2[$j]";
     	}
		$HTML_NETWORKTOPOLOGY_VAR=$HTML_NETWORKTOPOLOGY_VAR.'<TD>';
    	$HTML_NETWORKTOPOLOGY_VAR=$HTML_NETWORKTOPOLOGY_VAR.$RestStr;
       	$HTML_NETWORKTOPOLOGY_VAR=$HTML_NETWORKTOPOLOGY_VAR.'</TD>';
        
	$HTML_NETWORKTOPOLOGY_VAR=$HTML_NETWORKTOPOLOGY_VAR.'</TR>';
    }


    local(@keys);
    local(@values);
    @keys = keys %envHashTable;
    @values = values %envHashTable;
    while (@keys)
    {
        $HTML_ALLCOLLECTED_VAR=$HTML_ALLCOLLECTED_VAR.'<TR BGCOLOR="white" CLASS="TableRowColor">';

		$HTML_ALLCOLLECTED_VAR=$HTML_ALLCOLLECTED_VAR.'<TD ALIGN="right" VALIGN="top" WIDTH="1%"><FONT SIZE="-1">';
		$HTML_ALLCOLLECTED_VAR=$HTML_ALLCOLLECTED_VAR.pop(@keys);
		$HTML_ALLCOLLECTED_VAR =~ s/\n/'<BR>'/g;
		$HTML_ALLCOLLECTED_VAR=$HTML_ALLCOLLECTED_VAR.'</FONT></TD>';

		$HTML_ALLCOLLECTED_VAR=$HTML_ALLCOLLECTED_VAR.'<TD>';
		$HTML_ALLCOLLECTED_VAR=$HTML_ALLCOLLECTED_VAR.pop(@values);
		$HTML_ALLCOLLECTED_VAR =~ s/\n/'<BR>'/g;
		$HTML_ALLCOLLECTED_VAR=$HTML_ALLCOLLECTED_VAR.'</TD>';

	$HTML_ALLCOLLECTED_VAR=$HTML_ALLCOLLECTED_VAR.'</TR>';
    }

	local($temp);
    for ($i=0; $i<=$#FILESTOPACKAGE; $i++)
    {
		$temp="$ACTUALSNAPSHOTDIR/".basename($FILESTOPACKAGE[$i]);
		if (-e $temp)
		{
		    $temp=&getDetailsOnFile("$temp");
		    $HTML_ALLCOLLECTED_FILES=$HTML_ALLCOLLECTED_FILES.'<TR BGCOLOR="white" CLASS="TableRowColor">';
			$HTML_ALLCOLLECTED_FILES=$HTML_ALLCOLLECTED_FILES.'<TD>';
			#$HTML_ALLCOLLECTED_FILES=$HTML_ALLCOLLECTED_FILES.basename($FILESTOPACKAGE[$i]);
			$HTML_ALLCOLLECTED_FILES=$HTML_ALLCOLLECTED_FILES.'<A HREF="'.basename($FILESTOPACKAGE[$i]).'">'.basename($FILESTOPACKAGE[$i]).'</A>';
			$HTML_ALLCOLLECTED_FILES =~ s/\n/'<BR>'/g;
			$HTML_ALLCOLLECTED_FILES=$HTML_ALLCOLLECTED_FILES.'</TD>';
		
			$HTML_ALLCOLLECTED_FILES=$HTML_ALLCOLLECTED_FILES.'<TD>';
			$HTML_ALLCOLLECTED_FILES=$HTML_ALLCOLLECTED_FILES.$temp;
			$HTML_ALLCOLLECTED_FILES =~ s/\n/'<BR>'/g;
			$HTML_ALLCOLLECTED_FILES=$HTML_ALLCOLLECTED_FILES.'</TD>';
			$HTML_ALLCOLLECTED_FILES=$HTML_ALLCOLLECTED_FILES.'</TR>';
		}
    }

	if (&isArgument('-solve'))
	{
   		require MetronomeKB;import MetronomeKB;
		@values=solveProblems();
		for ($i=0; $i<=$#values; $i++)
		{
	    	$HTML_SUGGESTION_VAR=$HTML_SUGGESTION_VAR.'<TR BGCOLOR="white" CLASS="TableRowColor">';
			$HTML_SUGGESTION_VAR=$HTML_SUGGESTION_VAR.'<TD>';
			$HTML_SUGGESTION_VAR=$HTML_SUGGESTION_VAR.$values[$i];
			$HTML_SUGGESTION_VAR =~ s/\n/'<BR>'/g;
			$HTML_SUGGESTION_VAR=$HTML_SUGGESTION_VAR.'</TD>';
			$HTML_SUGGESTION_VAR=$HTML_SUGGESTION_VAR.'</TR>';
			$HTML_SUGGESTION_VAR=$HTML_SUGGESTION_VAR.'</TR>';
	 	}
 	}
 	else
	{$HTML_SUGGESTION_VAR='Disabled';}

    open (IN, "< $fileTemplate") || print "\n**** Impossibile aprire $fileTemplate.\n\n";
    open (OUT, "> $fileReport") || print "\n**** Impossibile creare $fileReport\n\n";

    while ($r = <IN>)
    {
    	$r =~ s/TIMESTAMP_VAR/$HTML_TIMESTAMP_VAR/g;
    	$r =~ s/MOSTIMPORTANT_VAR/$HTML_MOSTIMPORTANT_VAR/g;
		$r =~ s/ALLCOLLECTED_VAR/$HTML_ALLCOLLECTED_VAR/g;
		$r =~ s/ALLCOLLECTED_FILES/$HTML_ALLCOLLECTED_FILES/g;
		$r =~ s/NETWORKTOPOLOGY_VAR/$HTML_NETWORKTOPOLOGY_VAR/g;
		$r =~ s/SUGGESTION_VAR/$HTML_SUGGESTION_VAR/g;

	print OUT $r;
    }
    close(IN);
    close(OUT);

    print "[OK]\n";
}

# --------------------------------------------------
# input void
# output parameter void
# --------------------------------------------------
# sub printXMLReport()
# {
#     local($HTML);
#
#     local($fileReport);
#     local(@tokens);
#
#     print "writing xml report ...";

#     $fileReport=$ACTUALSNAPSHOTDIR.'/report.xml';
#     open (OUT, "> $fileReport") || print "\n**** Impossibile creare $fileReport\n\n";

#     #header
#     $HTML='<?xml version="1.0" encoding="UTF-8"?>'."\n\n";
#     print OUT $HTML;

#     $HTML='<MAESTRO_REPORT>'."\n";
#     print OUT $HTML;

#     $HTML='     <MOSTIMPORTANT_VAR>'."\n";
#     for ($i=0; $i<=$#MOSTIMPORTANT_VAR; $i++)
#     {
# 		$HTML=$HTML.'          <VAR name="'."$MOSTIMPORTANT_VAR[$i]".'">'."\n";
# 		$temp=$envHashTable{$MOSTIMPORTANT_VAR[$i]};
# 		$temp =~ s/\//\\/g;
# 		$HTML=$HTML."          $temp\n";
# 		$HTML=$HTML.'          </VAR>'."\n";
#     }
#     $HTML=$HTML.'     </MOSTIMPORTANT_VAR>'."\n";
#     print OUT $HTML;


#     $HTML='     <NETWORKTOPOLOGY>'."\n";
#     @tokens = split(/\n/,$envHashTable{$STR_VAR_CONMAN_SC});
#     for ($i=0; $i<=$#tokens; $i++)
#     {
#     	chop($tokens[$i]);
#
#     	local(@tokens2);
#         local($j);
#         local($limit);
#         local($RestStr);

# 	@tokens2 = split(/\s+/,"$tokens[$i]");

# 	$HTML=$HTML.'          <CPU name="'."$tokens2[0]".'" ';
#     	if ($tokens2[3] eq 'MASTER')
#     	{
#     		$HTML=$HTML.' isMaster="true"';
# 	}
# 	else
#     	{
#     		$HTML=$HTML.' isMaster="false"';
# 	}
# 	$HTML=$HTML.'>'."\n";
# 	$temp=$envHashTable{$MOSTIMPORTANT_VAR[$i]};
# 	$temp =~ s/\//\\/g;
# 	$temp =~ s/</#/g;
#
# 	$HTML=$HTML."          $temp\n";
# 	$HTML=$HTML.'          </CPU>'."\n";
#     }
#     $HTML=$HTML.'     </NETWORKTOPOLOGY>'."\n";
#     print OUT $HTML;

#     local(@keys);
#     local(@values);
#     @keys = keys %envHashTable;
#     @values = values %envHashTable;
#
#     $HTML='     <ALLCOLLECTED_VAR>'."\n";
#     while (@keys)
#     {
# 	$HTML=$HTML.'          <VAR name="'.pop(@keys).'">'."\n";
# 	$temp=pop(@values);
# 	$temp =~ s/\//\\/g;
# 	$HTML=$HTML."          ".$temp."\n";
# 	$HTML=$HTML.'          </VAR>'."\n";
#     }
#     $HTML=$HTML.'     </ALLCOLLECTED_VAR>'."\n";
#     print OUT $HTML;

#     $HTML='     <FILESTOPACKAGE>'."\n";
#     local($temp);
#     for ($i=0; $i<=$#FILESTOPACKAGE; $i++)
#     {
# 	$temp="$ACTUALSNAPSHOTDIR/".basename($FILESTOPACKAGE[$i]);
# 	if (-e $temp)
# 	{
# 	        $temp=&getDetailsOnFile("$temp");
# 		$temp =~ s/\//\\/g;
# 		$HTML=$HTML.'          <VAR name="'.basename($FILESTOPACKAGE[$i]).'">'."\n";
# 		$HTML=$HTML."          "."$temp"."\n";
# 		$HTML=$HTML.'          </VAR>'."\n";
#
# 	     	$HTML_ALLCOLLECTED_FILES=$HTML_ALLCOLLECTED_FILES.'<TR BGCOLOR="white" CLASS="TableRowColor">';
# 		$HTML_ALLCOLLECTED_FILES=$HTML_ALLCOLLECTED_FILES.'<TD>';
# 		#$HTML_ALLCOLLECTED_FILES=$HTML_ALLCOLLECTED_FILES.basename($FILESTOPACKAGE[$i]);
# 		$HTML_ALLCOLLECTED_FILES=$HTML_ALLCOLLECTED_FILES.'<A HREF="'.basename($FILESTOPACKAGE[$i]).'">'.basename($FILESTOPACKAGE[$i]).'</A>';
# 		$HTML_ALLCOLLECTED_FILES =~ s/\n/'<BR>'/g;
# 		$HTML_ALLCOLLECTED_FILES=$HTML_ALLCOLLECTED_FILES.'</TD>';
#
# 		$HTML_ALLCOLLECTED_FILES=$HTML_ALLCOLLECTED_FILES.'<TD>';
# 		$HTML_ALLCOLLECTED_FILES=$HTML_ALLCOLLECTED_FILES.$temp;
# 		$HTML_ALLCOLLECTED_FILES =~ s/\n/'<BR>'/g;
# 		$HTML_ALLCOLLECTED_FILES=$HTML_ALLCOLLECTED_FILES.'</TD>';
# 		$HTML_ALLCOLLECTED_FILES=$HTML_ALLCOLLECTED_FILES.'</TR>';
# 	}
#     }
#     $HTML=$HTML.'     </FILESTOPACKAGE>'."\n";
#     print OUT $HTML;

#     $HTML='</MAESTRO_REPORT>'."\n";
#     print OUT $HTML;
#     close(OUT);

#     print "[OK]\n";
# }


# --------------------------------------------------
# input String
# output String
# --------------------------------------------------
sub trim()
{
	local($tempStr);
	$tempStr=$_[0];

	# Replacing String
	$tempStr =~ s/^\s*//;
	$tempStr =~ s/\s*$//;

	return $tempStr;
}