###    	This script uses the "test" UNIX command to find files or directories.
### 	If the file is found, it can run an additional task which RC will
###	become this script RC. It is possible customizing the search setting a dead
###	line and a search rate time. The file will be searched immediately
###	at the script start and at the end of every search rate time. 
###	Using a parameter it is possible forcing this script RC if dead line is
###	reached without finding the file. Any way, it is possible suppressing the
###	dead line. Doing so the script ends only if the file if found.
###	The parameters this script need are distinguished using couples of
###	keywords. The first value is the option that is a constant identifying the
###	parameter, the second value is its argument that the script uses.
###	Here follows the list of all the option keyword: 
###
###	-kd	(mandatory)	it specify the option of the test UNIX command. See 
###				UNIX manuals to check all the allowed values.	
###			   
###	-fl	(mandatory)	it identify the searched file (or directory) path.
###
###	-dl	(mandatory)	it identify the deadline period (in seconds).
###
###	-nd	(mandatory)	this option (having no argument) suppress the dead line.
###				If using it the -dl option cannot be used (and vice	 
###				versa). 
###
###	-int	(mandatory)	it identify the search interval period (in seconds).
###	
###	-rc	(optional)	it identify the script RC when the dead line is reached
###				without finding the file (ignored if -nd is used).
###					
###	-tsk	(optional)	It identify the path of the task launched if the file
###				is found.
###
###	The couples option-argument can be passed in the order you want. 
###	This script execute check about the presence of the mandatory options, 
###	unknown options, options duplication, but NO VALIDATION checks are made on 
###	the arguments values. Here are reported two examples of the parameters
###	usage:
###
###		-kd f fl /music/angie -dl 30 -int 5 -rc 2527 -tsk /tools/kill
###
###	the script will check /music/angie path every 5 secs for a time of 30
###	secs. If this time will end without finding the file, then the script RC 
###	will be 2527. if the file will be found the task /tools/kill will be
### 	executed and its RC will become this script RC.
###
### 				-kd w -nd -int 5 -fl /music/angie
###
###	with those parameters a writable file  /music/angie will be searched every 
###	5 secs incessantly. 
### 	This script captures the scheduler variables, so when running using TWS:
### 	>>>>> SET THE $MAESTRO VARIABLE WITH THE TWS Home Directory Path <<<<<
###	The script provides a table reporting the passed options and information on
### 	the job and occurrence running it. If submitting it outside the scheduler, 
###	an automatic check is executed and the table is congruently modified.
###	When running it with the scheduler it is possible executing (with the -tsk 
###	option) single commands via <maestrohome>/jobmanrc.cmd, an example is:
###
###		-kd f nd int 10 fl /data/input.dat -tsk jobmanrc ls- l
###
###	If using this script with NT/2K OS note that:
### 	- to run this script it is necessary using a BASH shell (or an equivalent 
###	tool), so it is necessary adding to the PATH system variable the BASH 
###	libraries path. 
###	- If a BASH shell is needed to execute the additional task then it is
###	necessary enclosing the -tsk argument in single quotas, an example is:
###
###		-kd f nd int 10 fl /data/input.dat -tsk bash  task.bsh   
###	
###	The bash prefix can be added automatically (see the script code).   
###	- To make an optional task executing multiple Windows commands, it is
###	necessary including them in a .cmd file, an example is:
###
###		-kd f nd int 10 fl /data/input.dat -tsk command.cmd  
###
###	- When the optional task is not found, the script end with a RC=127.
###
###  Variable SetUp
OS=0
UNIX=1
NT=0
NT_OS_BASH="bash - "
HOLD_OUTPUT_STYLE=${MAESTRO_OUTPUT_STYLE}
MAESTRO_OUTPUT_STYLE=LONG
HOST=${UNISON_HOST}
#MAESTRO="/win32app/maestro"
MAESTRO="/ext/tws70a/maestro"
BASE_SCHED_NAME=${UNISON_SCHED}

total_args=12
RC_NO_COND=4
ERR_RC=4
int_time="-int"
tot_time="-dl"
NO_DEAD="-nd"
kind_check="-kd"
file_check="-fl"
task_check="-tsk"
rc_check="-rc"
will_die="yes"
end_msg="NO ACTION WAS TAKEN, EXITING WITH RC=$ERR_RC"
test1=0
test2=0
test3=0
test4=0
test5=0
test6=0


### define message and check subroutines

err_msg_more_than_1()
{
 echo "the option $@ has been specified more than one time"
 echo "$end_msg"
 exit $ERR_RC
}

err_msg_dead()
{
echo "the option $@ has been specified more than one time or "
echo "the $NO_DEAD and $tot_time options were both specified"
echo "$end_msg"
exit $ERR_RC
}

err_msg_mandatory_lack()
{
  echo "the mandatory option '$@' is missing"
  echo "$end_msg"
  exit $ERR_RC
}

check_argument()
{
  	if [ $@ -lt 2 ] 
       	 then        	 
       	  echo " the last option have no argument"
       	  echo "$end_msg"
          exit $ERR_RC
      fi
}

###
###
###
### Check if submitted via TWS 0=yes 1=no
###
test ${UNISON_STDLIST}
    submit_via_TWS=$?

###
### Check the max and the min arguments number
###
    
if [ $# -gt ${total_args} ]
    then
     echo "the number of the passed parameters is grater than the max value"
     echo "$end_msg"
     exit $ERR_RC
fi   

### The parse of the arguments

while [ $# -gt 0 ] 
 do
  temp=$1
  if [ "$temp" = "$NO_DEAD" ]
   then 
          
         will_die="no"
         test3=$(($test3 + 1))
         if [ $test3 -gt 1 ]
          then
           err_msg_dead "$1"
         fi
        shift
        temp=$1
  fi
    
  
if [ $# -ge 1 ]  
 then
  case $temp in
       "$kind_check")
         check_argument "$#"
         kind="$2"
         test1=$(($test1 + 1))
         if [ $test1 -gt 1 ]
         then
           err_msg_more_than_1 "$1"
         fi
         shift
         shift
        ;;
       "$file_check")
         check_argument "$#"
         file="$2"
         test2=$(($test2 + 1))
         if [ $test2 -gt 1 ] 
         then
           err_msg_more_than_1 "$1"
         fi
         shift
         shift
        ;;
       "$tot_time")
         check_argument "$#"
         time=$2
         timech="$2"
         test3=$(($test3 + 1))
         if [ $test3 -gt 1 ]
          then
           err_msg_dead "$1"
         fi
         shift
         shift
        ;;
       "$int_time")
         check_argument "$#"
         interval=$2
         test4=$(($test4 + 1))
         if [ $test4 -gt 1 ]
          then
           err_msg_more_than_1 "$1"
         fi
         shift
         shift
        ;;
       "$rc_check")
         check_argument "$#"
         RC_COND=$2
         test5=$(($test5 + 1))
         if [ $test5 -gt 1 ]
          then
           err_msg_more_than_1 "$1"
         fi
         shift
         shift
        ;;
       "$task_check")
         check_argument "$#"
         path_command="$2"
         test6=$(($test6 + 1))
         if [ $test6 -gt 1 ]
          then
           err_msg_more_than_1 "$1"
         fi
         shift
         shift
       ;;       
      *)
       echo "the option $1 is unknown"
       echo "$end_msg"
       exit $ERR_RC
      ;;
   esac
fi
done   

### Make checks about the mandatory parameters
   
if [ $test1 -ne 1 ]
 then
  err_msg_mandatory_lack "$kind_check"
fi

if [ $test2 -ne 1 ]
 then
  err_msg_mandatory_lack "$file_check"
fi

if [ $test3 -ne 1 ]
 then
  echo "the mandatory option '$tot_time' (or '$NO_DEAD') is missing"
  echo "$end_msg"
  exit $ERR_RC
fi

if [ $test4 -ne 1 ]
 then
    err_msg_mandatory_lack "$int_time"
fi

### Verify the the deadline time is a multiple of the 

if  [ $will_die = "yes" ] 
  then
    cycle_num=$(($time / $interval))
    time_trnc=$(($cycle_num * $interval))
     if [ $time -ne $time_trnc ]
      then 
       echo " '$tot_time' argument is smaller than the '$int_time' argument or is not a multiple"
       echo "$end_msg"
       exit $ERR_RC
     fi  
fi

### Set the variables used in the loop cycle

if [ $test6 -eq 0 ] && [ $test5 -eq 0 ] 
 then
  will_submit_job="no"
  will_cond_rc="no"
fi
 
if [ $test6 -eq 1 ] && [ $test5 -eq 1 ] 
 then
  will_submit_job="yes"
  will_cond_rc="yes"
fi

if [ $test6 -eq 0 ] && [ $test5 -eq 1 ]
 then
  will_submit_job="no"
  will_cond_rc="yes"
fi

if [ $test6 -eq 1 ] && [ $test5 -eq 0 ]
 then
  will_submit_job="yes"
  will_cond_rc="no"
fi
                       
###
### Set up command variables
###
  COMMAND="test -$kind $file"
  CD=" cd ${MAESTRO}/bin"
###
###
### Build cycle values
###
### Make addition settings due to 
if [ $will_die = "no" ] 
  then 
   deadline=1
  else  
   deadline=$(($time / $interval))
fi
 cycle=1
 index=$cycle
# fi
 sleep=$interval
 file_found="no"
###
### If windows bash environment build execution command
###
###  Removed, limits execution to bash only. Use 'bash - z:/a/b/c.sh' if bash is to be used
###
###if [ ${OS} -eq ${NT} ] && [ $test6 -eq 1 ]
###    then
###        echo "Setting execution to Windows/bash"
###        path_command="${NT_OS_BASH} ${path_command}"
###        
###fi 
###
### Echo variables to stdout
###
  echo "*********************************************************************************"
if [ ${submit_via_TWS} -eq 1 ]
 then
  echo "Job submtting from    =!!!!! Submitted outside TWS !!!!!"
 else 
  echo "Job submtting from    =${UNISON_JOB} on host=${UNISON_HOST}"
fi
  echo "Command arguments     =$kind"
  echo "File                  =$file"
  
if [ $will_die = "no" ] 
 then
  echo "Deadline time secs    =!!!! No Deadline has been set !!!!"
  echo "Deadline cycles       =!!!! No Deadline has been set !!!!"
 else
  echo "Deadline time secs    =$time"
  echo "Deadline cycles       =${deadline}"
fi
  echo "Check Cycle secs      =$interval" 
  echo "Task requested        =$path_command" 
if [ $test5 -eq 1 ] && [ $will_die = "no" ] 
 then
  echo "File not found RC     =${RC_COND} !!it is reduntant since there is no dead line!!"
 else
  echo "File not found RC     =${RC_COND}"
fi
if [ ${submit_via_TWS} -eq 1 ]
 then
  echo "Target host           =!!!!! Submitted outside TWS !!!!!"
  echo "base schedule         =!!!!! Submitted outside TWS !!!!!"
 else
  echo "Target host           =${HOST}"
  echo "base schedule         =${HOST}#${BASE_SCHED_NAME}"
fi
  echo "Command submitted     =${COMMAND}"
  echo "*********************************************************************************"
###
###
### Try to find $file immediately

 ${COMMAND} 
 RUN=$?
if [ ${RUN} -eq 0 ]
   then
     echo ""***File $file found immediately, check stopped""
     if [ "$will_submit_job" = "yes" ]
        then   
            ### Try to execute the assigned task 
            echo "!!!!! Attempting execution of ${path_command}"
            ###
            ### Execute if file found, then capture exit code from task
            ###
            ${path_command}
            RUN=$? 
            echo "task ${path_command} executed with a RC=${RUN}"
     fi 
   else
    
    ### Start of loop cycle
    ###

    while [ ${index} -le ${deadline} ]
     do
     sleep ${sleep}
        ${COMMAND} 
        RUN=$?

        ### Verify that the file exists
   
        if [ ${RUN} -eq 0 ]
         then
          msg_cycle=$((${cycle} * ${sleep}))
          file_found="yes"
           echo ""***File $file found after ${msg_cycle} seconds, check stopped""
          
           ### Try to execute the assigned task
        
             if [ "$will_submit_job" = "yes" ]
                then    
                    echo "!!!!! Attempting execution of ${path_command}"
                    ###
                    ### Execute if file found, then capture exit code from task
                    ###
                    ${path_command}
                    RUN=$? 
                    echo "task ${path_command} executed with a RC=${RUN}"
             fi 

              if [ $will_die = "no" ] 
                then 
                  index=2
              fi
            cycle=${deadline}
            sleep=0        
      fi 
    
    cycle=$(($cycle + 1))
    if [ $will_die = "yes" ] 
     then 
      index=$cycle
    fi
   done
###
### Check for run options
###

 if [ ${file_found} = "no" ]
    then
    echo "***File $file not found after $time seconds, check stopped" 
    RUN=${RC_NO_COND}
      if [ ${will_cond_rc} = "yes" ]
       then 
            echo "*** Conditioned RC is used ***"
            RUN=${RC_COND}
      fi
 fi
fi 
MAESTRO_OUTPUT_STYLE=${HOLD_OUTPUT_STYLE}
echo "exiting with a RC=${RUN}"
exit ${RUN}

###

