#!/bin/sh
################################################################################
#
# File:         proc.up.sh
# RCS:          @(#) $Header: /usr/local/SRC_CLEAR/maestro/JSS/maestro/OpC/RCS/proc.up.sh.sh,v 1.8.2.4 1999/05/06 21:59:17 pl Exp $
# Version:      5.2  1.2 
# Description:  Monitor the existance of a maestro process
#
#   	    	It does this by checking the existance of a file named
#               $MAEDIR/pids/<monitor>.pid
#
# Author:       ada@unison.com
# Language:     Bourne Shell
# Package:      Maestro/OpC
#
################################################################################

##
## OpC/proc.up.sh catalog definitions.
##

##
## message set id
##
MAE_OPC_PROCUP_MSGSET=227

##
## message ids
##
MSG_USAGE_1=1
MSG_USAGE_2=2
MSG_USAGE_3=3
MSG_USAGE_4=4
MSG_USAGE_5=5
MSG_USAGE_6=6

# Set up some defaults
MACH_TYPE=`uname -s`
PROGNAME=`basename $0`
MAEDIR=/usr/lib/maestro
PIDDIR=${MAEDIR}/pids

#
# Set LANG to "C", if it is not set or set to "", as catopen does
# not assume a default for %L specified in the template for NLSPATH.
#
if [ -z "$LANG" ]
then
  LANG="C"; export LANG
fi

NLSPATH=`pwd`/catalog/%L/%N.cat:$NLSPATH; export NLSPATH

MECHO=`maestro`/bin/mecho; export MECHO

############################################################
# Usage
#

usage () 
{
 echo "`$MECHO $MAE_OPC_PROCUP_MSGSET $MSG_USAGE_1 ${PROGNAME}`"  >&2
 echo "`$MECHO $MAE_OPC_PROCUP_MSGSET $MSG_USAGE_2`"              >&2
 echo "`$MECHO $MAE_OPC_PROCUP_MSGSET $MSG_USAGE_3`"              >&2
 echo "`$MECHO $MAE_OPC_PROCUP_MSGSET $MSG_USAGE_4`"              >&2
 echo "`$MECHO $MAE_OPC_PROCUP_MSGSET $MSG_USAGE_5`"              >&2
 echo "`$MECHO $MAE_OPC_PROCUP_MSGSET $MSG_USAGE_6`"              >&2
 exit 1
}

############################################################
# Change default variables according to machine type.
#

arch_defaults () 
{
 case ${MACH_TYPE} in
  HP-UX)
	os_version=`uname -r | awk -F. '{ print $2 }'`
	case $os_version in
 	    09)
 	        OPCMON=/usr/OV/bin/OpC/opcmon
		;;
	    10)
		OPCMON=/opt/OV/bin/OpC/opcmon
		;;
	esac

        PID_CMD=`ps -ef | grep ${PROCESS} | grep -v grep | grep -v $PROGNAME | awk '{print $2}'`
    	TST_CMD="ps -p `cat ${PIDDIR}/${MONITOR}.pid 2> /dev/null` > /dev/null 2>&1"
	;;

  SunOS)
	os_version=`uname -r | awk -F. '{ print $1 }'`
	case $os_version in
	    4) 
		OPCMON=/usr/OV/bin/OpC/opcmon
		;;
	    5)
		OPCMON=/opt/OV/bin/OpC/opcmon
		;;
        esac

        PID_CMD=`ps -e | grep ${PROCESS} | grep -v grep | grep -v $PROGNAME | awk '{print $2}'`
    	TST_CMD="ps -p `cat ${PIDDIR}/${MONITOR}.pid 2> /dev/null` > /dev/null 2>&1"
	;;

  AIX)
	OPCMON=/usr/lpp/OV/OpC/opcmon
        PID_CMD=`ps -e | grep ${PROCESS} | grep -v grep | grep -v $PROGNAME | awk '{print $2}'`
    	TST_CMD="ps -p `cat ${PIDDIR}/${MONITOR}.pid 2> /dev/null` > /dev/null 2>&1"
        ;;
 esac
 return 0
}


############################################################
# Main thread
#

 if [ "$1" = "-ownpid" ]
 then
   ownpid=1
   shift
 else
   ownpid=0
 fi

 if [ $# -ne 2 -o "$1" = "-?" -o "$1" = "-h" ]; then
   usage
 fi

 PROCESS=$1
 MONITOR=$2

 arch_defaults
 if [ -s ${PIDDIR}/"${MONITOR}".pid ]
 then
    if eval $TST_CMD
    then
     ${OPCMON} "${MONITOR}=1"
    else
     ${OPCMON} "${MONITOR}=0"
     rm ${PIDDIR}/"${MONITOR}".pid
    fi
  elif [ $ownpid -eq 1 ]
  then
    ${OPCMON} "${MONITOR}=1"
  else  
    PID=$PID_CMD
    if [ -z "$PID" ]
    then
      ${OPCMON} "${MONITOR}=0"      
    else
      ${OPCMON} "${MONITOR}=1"
      echo $PID > ${PIDDIR}/"${MONITOR}".pid
    fi
 fi
 exit 0

################################################################################
# End of file: proc.up.sh
################################################################################
